#### Function for estimating mean under lamda using the Bivariate Normal Pattern Mixture Model ####

estimate<-function(x.0, x.1, y.0, lamda, draws)
{
	k<-draws
	n.0<-length(x.0)
	n.1<-length(x.1)
	xx.0<-x.0+lamda*y.0
	s11.0<-sum((x.0-mean(x.0))^2)/n.0
	s11.1<-sum((x.1-mean(x.1))^2)/n.1	
	s22.0<-sum((y.0-mean(y.0))^2)/n.0
	s33.0<-sum((xx.0-mean(xx.0))^2)/n.0	
	s12.0<-sum((x.0-mean(x.0))*(y.0-mean(y.0)))/n.0
	s13.0<-sum((x.0-mean(x.0))*(xx.0-mean(xx.0)))/n.0
    	s221.0<-s22.0-(s12.0)^2/s11.0
   	s112.0<-s11.0-(s12.0)^2/s22.0
    	s113.0<-s11.0-(s13.0)^2/s33.0

	if (lamda==0)
	{
		pi.0<-rbeta(k,n.0+0.5,n.1+0.5)
		sigma11.0<-(n.0*s11.0)/rchisq(k,n.0-1)
		sigma11.1<-(n.1*s11.1)/rchisq(k,n.1-1)
		mu1.1<-rnorm(k,mean(x.1),sqrt(sigma11.1/n.1))
		mu1.0<-rnorm(k,mean(x.0),sqrt(sigma11.0/n.0))
		sigma221.0<-(n.0*s221.0)/rchisq(k,n.0-2)
		slope<-rnorm(k,s12.0/s11.0,sqrt(sigma221.0/(n.0*s11.0)))
		int<-rnorm(k,mean(y.0)-slope*mean(x.0),sqrt(sigma221.0/n.0))
		mu2<-pi.0*(int+slope*mu1.0)+(1-pi.0)*(int+slope*mu1.1)
	}

	stop<-0
	if (lamda>0 & lamda<Inf)
	{
		pi.0<-rbeta(k,n.0+0.5,n.1+0.5)
		sigma33.0<-(n.0*s33.0)/rchisq(k,n.0-1)
		sigma11.1<-(n.1*s11.1)/rchisq(k,n.1-1)
		sigma113.0<-(n.0*s113.0)/rchisq(k,n.0-2)
		while(sum(sigma113.0>sigma11.1)>0 & stop<100)
		{
			index<-which(sigma113.0>sigma11.1)
			sigma11.1[index]<-(n.1*s11.1)/rchisq(length(index),n.1-1)
			sigma113.0[index]<-(n.0*s113.0)/rchisq(length(index),n.0-2)
			stop<-stop+1
		}
		slope<-rnorm(k,s13.0/s33.0,sqrt(sigma113.0/(n.0*s33.0)))
		int<-rnorm(k,mean(x.0)-slope*mean(xx.0),sqrt(sigma113.0/n.0))
		mu3.0<-rnorm(k,mean(xx.0),sqrt(sigma33.0/n.0))
		mu1.1<-rnorm(k,mean(x.1),sqrt(sigma11.1/n.1))
		mu1.0<-int+mu3.0*slope
		mu1<-pi.0*mu1.0+(1-pi.0)*mu1.1
		mu3<-pi.0*mu3.0+(1-pi.0)*(mu1.1-int)/slope
		mu2<-(mu3-mu1)/lamda
	}

	if (lamda==Inf)
	{
		pi.0<-rbeta(k,n.0+0.5,n.1+0.5)
		sigma22.0<-(n.0*s22.0)/rchisq(k,n.0-1)
		sigma11.1<-(n.1*s11.1)/rchisq(k,n.1-1)
 		sigma112.0<-(n.0*s112.0)/rchisq(k,n.0-2)
		while(sum(sigma112.0>sigma11.1)>0 & stop<100)
		{
			index<-which(sigma112.0>sigma11.1)
			sigma11.1[index]<-(n.1*s11.1)/rchisq(length(index),n.1-1)
			sigma112.0[index]<-(n.0*s112.0)/rchisq(length(index),n.0-2)
			stop<-stop+1
		}
		slope<-rnorm(k,s12.0/s22.0,sqrt(sigma112.0/(n.0*s22.0)))
		int<-rnorm(k,mean(x.0)-slope*mean(y.0),sqrt(sigma112.0/n.0))
		mu2.0<-rnorm(k,mean(y.0),sqrt(sigma22.0/n.0))
		mu1.1<-rnorm(k,mean(x.1),sqrt(sigma11.1/n.1))
		mu2.1<-(mu1.1-int)/slope
		mu2<-pi.0*mu2.0+(1-pi.0)*mu2.1
	}
		
	return(mu2)
}


estimate2<-function(x.0, x.1, y.0, lamda, draws)
{
	k<-draws
	n.0<-length(x.0)
	n.1<-length(x.1)
	xx.0<-x.0+lamda*y.0
	s11.0<-sum((x.0-mean(x.0))^2)/n.0
	s11.1<-sum((x.1-mean(x.1))^2)/n.1	
	s22.0<-sum((y.0-mean(y.0))^2)/n.0
	s33.0<-sum((xx.0-mean(xx.0))^2)/n.0	
	s12.0<-sum((x.0-mean(x.0))*(y.0-mean(y.0)))/n.0
	s13.0<-sum((x.0-mean(x.0))*(xx.0-mean(xx.0)))/n.0
    	s221.0<-s22.0-(s12.0)^2/s11.0
   	s112.0<-s11.0-(s12.0)^2/s22.0
    	s113.0<-s11.0-(s13.0)^2/s33.0

	if (lamda==0)
	{
		pi.0<-rbeta(k,n.0+0.5,n.1+0.5)
		sigma11.0<-(n.0*s11.0)/rchisq(k,n.0-1)
		sigma11.1<-(n.1*s11.1)/rchisq(k,n.1-1)
		mu1.1<-rnorm(k,mean(x.1),sqrt(sigma11.1/n.1))
		mu1.0<-rnorm(k,mean(x.0),sqrt(sigma11.0/n.0))
		sigma221.0<-(n.0*s221.0)/rchisq(k,n.0-2)
		slope<-rnorm(k,s12.0/s11.0,sqrt(sigma221.0/(n.0*s11.0)))
		int<-rnorm(k,mean(y.0)-slope*mean(x.0),sqrt(sigma221.0/n.0))
		mu2.0<-int+slope*mu1.0
		mu2.1<-int+slope*mu1.1
		mu2<-pi.0*(int+slope*mu1.0)+(1-pi.0)*(int+slope*mu1.1)
		sigma22.0<-sigma221.0+sigma11.0*slope^2
		sigma22.1<-sigma221.0+sigma11.1*slope^2
		sigma12.1<-slope*sigma11.1
	}

	stop<-0
	if (lamda>0 & lamda<Inf)
	{
		pi.0<-rbeta(k,n.0+0.5,n.1+0.5)
		sigma33.0<-(n.0*s33.0)/rchisq(k,n.0-1)
		sigma11.1<-(n.1*s11.1)/rchisq(k,n.1-1)
		sigma113.0<-(n.0*s113.0)/rchisq(k,n.0-2)
		while(sum(sigma113.0>sigma11.1)>0 & stop<100)
		{
			index<-which(sigma113.0>sigma11.1)
			sigma11.1[index]<-(n.1*s11.1)/rchisq(length(index),n.1-1)
			sigma113.0[index]<-(n.0*s113.0)/rchisq(length(index),n.0-2)
			stop<-stop+1
		}
		slope<-rnorm(k,s13.0/s33.0,sqrt(sigma113.0/(n.0*s33.0)))
		int<-rnorm(k,mean(x.0)-slope*mean(xx.0),sqrt(sigma113.0/n.0))
		mu3.0<-rnorm(k,mean(xx.0),sqrt(sigma33.0/n.0))
		mu1.1<-rnorm(k,mean(x.1),sqrt(sigma11.1/n.1))
		mu1.0<-int+mu3.0*slope
		mu1<-pi.0*mu1.0+(1-pi.0)*mu1.1
		mu3.1<-(mu1.1-int)/slope
		mu3<-pi.0*mu3.0+(1-pi.0)*(mu1.1-int)/slope
		mu2<-(mu3-mu1)/lamda
		mu2.0<-(mu3.0-mu1.0)/lamda
		mu2.1<-(mu3.1-mu1.1)/lamda
		sigma11.0<-sigma113.0+sigma33.0*slope^2
		sigma33.1<-(sigma11.1-sigma113.0)/(slope^2)
		sigma13.0<-slope*sigma33.0
		sigma13.1<-slope*sigma33.1
		sigma22.0<-(1/lamda^2)*(sigma33.0+sigma11.0-2*sigma13.0)
		sigma22.1<-(1/lamda^2)*(sigma33.1+sigma11.1-2*sigma13.1)
		sigma12.1<-1/(2*lamda)*(sigma33.1-sigma11.1-sigma22.1*lamda^2)
	}

	if (lamda==Inf)
	{
		pi.0<-rbeta(k,n.0+0.5,n.1+0.5)
		sigma22.0<-(n.0*s22.0)/rchisq(k,n.0-1)
		sigma11.1<-(n.1*s11.1)/rchisq(k,n.1-1)
 		sigma112.0<-(n.0*s112.0)/rchisq(k,n.0-2)
		while(sum(sigma112.0>sigma11.1)>0 & stop<100)
		{
			index<-which(sigma112.0>sigma11.1)
			sigma11.1[index]<-(n.1*s11.1)/rchisq(length(index),n.1-1)
			sigma112.0[index]<-(n.0*s112.0)/rchisq(length(index),n.0-2)
			stop<-stop+1
		}
		slope<-rnorm(k,s12.0/s22.0,sqrt(sigma112.0/(n.0*s22.0)))
		int<-rnorm(k,mean(x.0)-slope*mean(y.0),sqrt(sigma112.0/n.0))
		mu2.0<-rnorm(k,mean(y.0),sqrt(sigma22.0/n.0))
		mu1.1<-rnorm(k,mean(x.1),sqrt(sigma11.1/n.1))
		mu2.1<-(mu1.1-int)/slope
		mu2<-pi.0*mu2.0+(1-pi.0)*mu2.1
		sigma22.1<-(sigma11.1-sigma112.0)/(slope^2)
		sigma12.1<-slope*sigma22.1
	}
	sigma22.1[sigma22.1<0]<-0
	return(list(pi=pi.0, mu1.1=mu1.1, mu2.0=mu2.0, mu2.1=mu2.1, sigma11.1=sigma11.1, sigma12.1=sigma12.1, sigma22.0=sigma22.0, sigma22.1=sigma22.1))
}


estimate3<-function(x.0, x.1, y.0, lamda, draws)
{
	k<-draws
	n.0<-length(x.0)
	n.1<-length(x.1)
	xx.0<-x.0+lamda*y.0
	s11.0<-sum((x.0-mean(x.0))^2)/n.0
	s11.1<-sum((x.1-mean(x.1))^2)/n.1	
	s22.0<-sum((y.0-mean(y.0))^2)/n.0
	s33.0<-sum((xx.0-mean(xx.0))^2)/n.0	
	s12.0<-sum((x.0-mean(x.0))*(y.0-mean(y.0)))/n.0
	s13.0<-sum((x.0-mean(x.0))*(xx.0-mean(xx.0)))/n.0
    	s221.0<-s22.0-(s12.0)^2/s11.0
   	s112.0<-s11.0-(s12.0)^2/s22.0
    	s113.0<-s11.0-(s13.0)^2/s33.0

	if (lamda==0)
	{
		pi.0<-rbeta(k,n.0+0.5,n.1+0.5)
		sigma11.0<-(n.0*s11.0)/rchisq(k,n.0-1)
		sigma11.1<-(n.1*s11.1)/rchisq(k,n.1-1)
		mu1.1<-rnorm(k,mean(x.1),sqrt(sigma11.1/n.1))
		mu1.0<-rnorm(k,mean(x.0),sqrt(sigma11.0/n.0))
		sigma221.0<-(n.0*s221.0)/rchisq(k,n.0-2)
		slope<-rnorm(k,s12.0/s11.0,sqrt(sigma221.0/(n.0*s11.0)))
		int<-rnorm(k,mean(y.0)-slope*mean(x.0),sqrt(sigma221.0/n.0))
		mu2.0<-int+slope*mu1.0
		mu2.1<-int+slope*mu1.1
		mu2<-pi.0*(int+slope*mu1.0)+(1-pi.0)*(int+slope*mu1.1)
		sigma22.0<-sigma221.0+sigma11.0*slope^2
		sigma22.1<-sigma221.0+sigma11.1*slope^2
		sigma12.1<-slope*sigma11.1
	}

	stop<-0
	if (lamda>0 & lamda<Inf)
	{
		pi.0<-rbeta(k,n.0+0.5,n.1+0.5)
		sigma33.0<-(n.0*s33.0)/rchisq(k,n.0-1)
		sigma11.1<-(n.1*s11.1)/rchisq(k,n.1-1)
		sigma113.0<-(n.0*s113.0)/rchisq(k,n.0-2)
		while(sum(sigma113.0>sigma11.1)>0 & stop<100)
		{
			index<-which(sigma113.0>sigma11.1)
			sigma11.1[index]<-(n.1*s11.1)/rchisq(length(index),n.1-1)
			sigma113.0[index]<-(n.0*s113.0)/rchisq(length(index),n.0-2)
			stop<-stop+1
		}
		slope<-rnorm(k,s13.0/s33.0,sqrt(sigma113.0/(n.0*s33.0)))
		int<-rnorm(k,mean(x.0)-slope*mean(xx.0),sqrt(sigma113.0/n.0))
		mu3.0<-rnorm(k,mean(xx.0),sqrt(sigma33.0/n.0))
		mu1.1<-rnorm(k,mean(x.1),sqrt(sigma11.1/n.1))
		mu1.0<-int+mu3.0*slope
		mu1<-pi.0*mu1.0+(1-pi.0)*mu1.1
		mu3.1<-(mu1.1-int)/slope
		mu3<-pi.0*mu3.0+(1-pi.0)*(mu1.1-int)/slope
		mu2<-(mu3-mu1)/lamda
		mu2.0<-(mu3.0-mu1.0)/lamda
		mu2.1<-(mu3.1-mu1.1)/lamda
		sigma11.0<-sigma113.0+sigma33.0*slope^2
		sigma33.1<-(sigma11.1-sigma113.0)/(slope^2)
		sigma13.0<-slope*sigma33.0
		sigma13.1<-slope*sigma33.1
		sigma22.0<-(1/lamda^2)*(sigma33.0+sigma11.0-2*sigma13.0)
		sigma22.1<-(1/lamda^2)*(sigma33.1+sigma11.1-2*sigma13.1)
		sigma12.1<-1/(2*lamda)*(sigma33.1-sigma11.1-sigma22.1*lamda^2)
	}

	if (lamda==Inf)
	{
		pi.0<-rbeta(k,n.0+0.5,n.1+0.5)
		sigma22.0<-(n.0*s22.0)/rchisq(k,n.0-1)
		sigma11.1<-(n.1*s11.1)/rchisq(k,n.1-1)
 		sigma112.0<-(n.0*s112.0)/rchisq(k,n.0-2)
		while(sum(sigma112.0>sigma11.1)>0 & stop<100)
		{
			index<-which(sigma112.0>sigma11.1)
			sigma11.1[index]<-(n.1*s11.1)/rchisq(length(index),n.1-1)
			sigma112.0[index]<-(n.0*s112.0)/rchisq(length(index),n.0-2)
			stop<-stop+1
		}
		slope<-rnorm(k,s12.0/s22.0,sqrt(sigma112.0/(n.0*s22.0)))
		int<-rnorm(k,mean(x.0)-slope*mean(y.0),sqrt(sigma112.0/n.0))
		mu2.0<-rnorm(k,mean(y.0),sqrt(sigma22.0/n.0))
		mu1.1<-rnorm(k,mean(x.1),sqrt(sigma11.1/n.1))
		mu2.1<-(mu1.1-int)/slope
		mu2<-pi.0*mu2.0+(1-pi.0)*mu2.1
		sigma22.1<-(sigma11.1-sigma112.0)/(slope^2)
		sigma12.1<-slope*sigma22.1
	}
		
	return(cbind(int, slope))
}

#### Function for Gibbs sampling for penalized spline model ####

#Samples model parameters using Gibbs/Metropolis Algorithm
#X=an n by p+q data matrix including p colums of intercept, covariates, and propensity; and q columns of propensity splines
#Y=outcome
#q=number of knots
#Returns an (p+q+4) by S-1000 matrix of regression parameters, variance components, alpha, and rejection rates for alpha

#Homoscedastic errors
pspp.hom<-function(X, Y, q, S)
{
	#Define functions for Gibbs sampling
	sample.beta<-function(X, Y, sigma2.mu, sigma2.ep, alpha, q)
	{
		p<-(ncol(X)-q)
		D<-rbind(cbind(diag(0,p),matrix(0,ncol=q,nrow=p)),cbind(matrix(0,ncol=p,nrow=q),diag(1,q)))
		post.var<-sigma2.ep*solve(t(X)%*%X+(sigma2.ep/sigma2.mu)*D)
		post.mean<-solve(t(X)%*%X+(sigma2.ep/sigma2.mu)*D)%*%t(X)%*%Y
		beta<-mvrnorm(1,post.mean,post.var)
		return(beta)
	}

	sample.sigma2.mu<-function(X, beta, Au, Bu, q)
	{	
		u<-beta[(ncol(X)-q+1):ncol(X)]
		sigma2.mu<-1/rgamma(1,Au+q/2,Bu+sum(u^2)/2)
		return(sigma2.mu)
	}

	sample.sigma2.ep<-function(X, Y, beta, alpha, Ae, Be)
	{	
		ssr<-t(Y-X%*%beta)%*%(Y-X%*%beta)
		sigma2.ep<-1/rgamma(1,Ae+length(Y)/2,Be+ssr/2)
		return(sigma2.ep)
	}

	#Initialize
	if (S<2000) {S<-2000}
	Au<-Bu<-Ae<-Be<-0.0001
	BETA<-matrix(rep(0, ncol(X)))
	SIGMA2.MU<-SIGMA2.EP<-1
	ALPHA<-AR<-0
	PRED<-NA

	#Gibbs sampler with Metropolis-Hastings algorithm for alpha (for heteroscedastic errors)
	for(i in 2:S)
	{
		ALPHA<-c(ALPHA, 0)
		AR<-c(AR, 0)
		BETA<-cbind(BETA, sample.beta(X, Y, SIGMA2.MU[i-1], SIGMA2.EP[i-1], ALPHA[i], q))
		SIGMA2.MU<-c(SIGMA2.MU, sample.sigma2.mu(X, BETA[,i], Au, Bu, q))
		SIGMA2.EP<-c(SIGMA2.EP, sample.sigma2.ep(X, Y, BETA[,i], ALPHA[i], Ae, Be))
	}

	#Returns parameter estimates, variance components, alpha, and acceptance rate for alpha
	return(list("BETA"=BETA[,1001:S], "SIGMA2"=SIGMA2.EP[1001:S], "ALPHA"=ALPHA[1001:S], "AR"=AR[1001:S]))

}

