##################################### Load Required Packages #####################################

library(MASS)
library(snowfall)
cpus<-4
sfInit(cpus=cpus,type="SOCK",parallel=T)
sfClusterSetupRNG(type="RNGstream",seed=as.numeric(Sys.time()))
sfLibrary(MASS)

#bd: before deletion
#cc: complete case analysis
#ps: penalized spline analysis
#ml: pattern mixture analysis via maximum likelihood
#bc: pattern mixture analysis via bayes
#outputs bias (bias), rmse (rmse), confidence interval width (ciw), and coverage rate (cov)
names<-c("bias.bd", "bias.cc", "bias.ps.0", "bias.ml.0", "bias.bc.0", "bias.ps.1", "bias.ml.1", "bias.bc.1", "bias.ps.n", "bias.ml.n", "bias.bc.n", 
         "rmse.bd", "rmse.cc", "rmse.ps.0", "rmse.ml.0", "rmse.bc.0", "rmse.ps.1", "rmse.ml.1", "rmse.bc.1", "rmse.ps.n", "rmse.ml.n", "rmse.bc.n", 
	   "ciw.bd", "ciw.cc", "ciw.ps.0", "ciw.ml.0", "ciw.bc.0", "ciw.ps.1", "ciw.ml.1", "ciw.bc.1", "ciw.ps.n", "ciw.ml.n", "ciw.bc.n", 
	   "cov.bd", "cov.cc", "cov.ps.0", "cov.ml.0", "cov.bc.0", "cov.ps.1", "cov.ml.1", "cov.bc.1", "cov.ps.n", "cov.ml.n", "cov.bc.n")

names.tr<-c("bias.bd", "bias.cc", "bias.tr", "bias.ps.0", "bias.ml.0", "bias.bc.0", "bias.ps.1", "bias.ml.1", "bias.bc.1", "bias.ps.n", "bias.ml.n", "bias.bc.n", 
            "rmse.bd", "rmse.cc", "rmse.tr", "rmse.ps.0", "rmse.ml.0", "rmse.bc.0", "rmse.ps.1", "rmse.ml.1", "rmse.bc.1", "rmse.ps.n", "rmse.ml.n", "rmse.bc.n", 
	      "ciw.bd", "ciw.cc", "ciw.tr", "ciw.ps.0", "ciw.ml.0", "ciw.bc.0", "ciw.ps.1", "ciw.ml.1", "ciw.bc.1", "ciw.ps.n", "ciw.ml.n", "ciw.bc.n", 
	      "cov.bd", "cov.cc", "cov.tr", "cov.ps.0", "cov.ml.0", "cov.bc.0", "cov.ps.1", "cov.ml.1", "cov.bc.1", "cov.ps.n", "cov.ml.n", "cov.bc.n")

#Set replications
M<-1000


######################################## Scenario 1: Bivariate normal ########################################

#### Lamda=0 ####

sim1<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
m<-rbinom(n,1,0.5)
n.0<-sum(m==0)
n.1<-sum(m==1)
XY<-mvrnorm(n.0,rep(0,2),matrix(c(1,0.5,0.5,1),2,2))
X.0<-XY[,1]
X.1<-rnorm(n.1,1,1)
Y.0<-XY[,2]
X<-c(X.0,X.1)
true.mu2<-0.25
#true.mu2<-0.5
#true.mu2<-1
data<-data.frame(X=X, Y=c(Y.0,rep(NA,n.1)), m=c(rep(0,n.0),rep(1,n.1)))


#### BD & CC ####

mean.bd<-0
var.bd<-0
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	X<-data$X
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	X.0<-X[m==0]
	X.1<-X[m==1]
	Y.0<-Y[m==0]
	X<-c(X.0,X.1)

	b.lamda<-function(lamda)
	{
		if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
		if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
		return(b)
	}

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=1 ####

sim2<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
m<-rbinom(n,1,0.5)
n.0<-sum(m==0)
n.1<-sum(m==1)
XY<-mvrnorm(n.0,rep(0,2),matrix(c(1,0.5,0.5,1),2,2))
X.0<-XY[,1]
X.1<-rnorm(n.1,1,1)
Y.0<-XY[,2]
X<-c(X.0,X.1)
#true.mu2<-0.25
true.mu2<-0.5
#true.mu2<-1
data<-data.frame(X=X, Y=c(Y.0,rep(NA,n.1)), m=c(rep(0,n.0),rep(1,n.1)))


#### BD & CC ####

mean.bd<-0
var.bd<-0
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	X<-data$X
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	X.0<-X[m==0]
	X.1<-X[m==1]
	Y.0<-Y[m==0]
	X<-c(X.0,X.1)

	b.lamda<-function(lamda)
	{
		if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
		if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
		return(b)
	}

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=Inf ####

sim3<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
m<-rbinom(n,1,0.5)
n.0<-sum(m==0)
n.1<-sum(m==1)
XY<-mvrnorm(n.0,rep(0,2),matrix(c(1,0.5,0.5,1),2,2))
X.0<-XY[,1]
X.1<-rnorm(n.1,1,1)
Y.0<-XY[,2]
X<-c(X.0,X.1)
#true.mu2<-0.25
#true.mu2<-0.5
true.mu2<-1
data<-data.frame(X=X, Y=c(Y.0,rep(NA,n.1)), m=c(rep(0,n.0),rep(1,n.1)))


#### BD & CC ####

mean.bd<-0
var.bd<-0
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	X<-data$X
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	X.0<-X[m==0]
	X.1<-X[m==1]
	Y.0<-Y[m==0]
	X<-c(X.0,X.1)

	b.lamda<-function(lamda)
	{
		if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
		if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
		return(b)
	}

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


######################################## Scenario 2: Bivariate non-normal ########################################

#### Lamda=0 ####

sim4<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
X<-4*rgamma(n,1,1)
Y<-rnorm(n,10+X,1)
U<-rnorm(n,-1.5+0.5*X,1)
#U<-rnorm(n,-2.5+0.15*(X+Y),1)
#U<-rnorm(n,-3.5+0.25*Y,1)
m<-(U>0)
n.0<-sum(m==0)
n.1<-sum(m==1)
X.0<-X[m==0]
X.1<-X[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
U.0<-U[m==0]
U.1<-U[m==1]
true.mu2<-14
data<-data.frame(X=X, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n.0

#### True model ####

Z.0<-cbind(rep(1,n.0),X.0)
Z.1<-cbind(rep(1,n.1),X.1)

S<-1000
s2<-t(Y.0-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0)%*%(Y.0-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0)/(n.0-ncol(Z.0))
imp<-NULL

#Generate proxies
for (i in 1:S)
{
	sigma2<-1/rgamma(1,(n.0-ncol(Z.0))/2,(n.0-ncol(Z.0))*s2/2)
	coef<-mvrnorm(1,solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0,solve(t(Z.0)%*%Z.0)*sigma2)
	imp<-cbind(imp, rnorm(n.1,Z.1%*%coef,sqrt(sigma2)))
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.tr<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.tr<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
	

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(X.0, probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(W.0, probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(Y.0, probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	X<-data$X
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	X.0<-X[m==0]
	X.1<-X[m==1]
	Y.0<-Y[m==0]
	X<-c(X.0,X.1)

	b.lamda<-function(lamda)
	{
		if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
		if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
		return(b)
	}

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.tr<-mean.tr-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.tr<-bias.tr^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.tr<-2*t*sqrt(var.tr)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.tr<-(true.mu2<(mean.tr+t*sqrt(var.tr)) & true.mu2>(mean.tr-t*sqrt(var.tr)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.tr, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.tr, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.tr, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.tr, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=1 ####

sim5<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
X<-4*rgamma(n,1,1)
Y<-rnorm(n,10+X,1)
#U<-rnorm(n,-1.5+0.5*X,1)
U<-rnorm(n,-2.5+0.15*(X+Y),1)
#U<-rnorm(n,-3.5+0.25*Y,1)
m<-(U>0)
n.0<-sum(m==0)
n.1<-sum(m==1)
X.0<-X[m==0]
X.1<-X[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
U.0<-U[m==0]
U.1<-U[m==1]
true.mu2<-14
data<-data.frame(X=X, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n.0

#### True model ####

W.0<-Y.0+X.0
coef<-lm(W.0~X.0)$coefficients
W.new<-as.matrix(coef[1]+coef[2]*X.1)
a0<-qnorm(mean(m),0,1)-mean(c(W.0,W.new))
a1<-1
sigma2<-1
U.new.0<-as.matrix(rnorm(n.0,a0+a1*W.0,1))
U.new.1<-as.matrix(rnorm(n.1,a0+a1*W.new,1))

for (i in 1:1000)
{
	j<-0
	while(max(U.new.0[,i])>0)
	{
		index<-which(U.new.0[,i]>0)
		U.new.0[,i][index]<-rnorm(length(index),a0+a1*W.0[index],sqrt(sigma2))
		j<-j+1
		if (j==10000) {U.new.0[,i][U.new.0[,i]>0]<-0}
	}

	j<-0
	while(min(U.new.1[,i])<0)
	{
		index<-which(U.new.1[,i]<0)
		U.new.1[,i][index]<-rnorm(length(index),a0+a1*W.new[,i][index],sqrt(sigma2))
		j<-j+1
		if (j==10000) {U.new.1[,i][U.new.1[,i]<0]<-0}
	}

	U.new<-c(U.new.0[,i],U.new.1[,i])

	UX.0<-cbind(rep(1,n.0),U.new.0[,i],X.0)
	UX.1<-cbind(rep(1,n.1),U.new.1[,i],X.1)
	s2<-t(W.0-UX.0%*%solve(t(UX.0)%*%UX.0)%*%t(UX.0)%*%W.0)%*%(W.0-UX.0%*%solve(t(UX.0)%*%UX.0)%*%t(UX.0)%*%W.0)/(n.0-ncol(UX.0))
	sigma2<-1/rgamma(1,(n.0-ncol(UX.0))/2,(n.0-ncol(UX.0))*s2/2)
	coef<-mvrnorm(1,solve(t(UX.0)%*%UX.0)%*%t(UX.0)%*%W.0,solve(t(UX.0)%*%UX.0)*sigma2)
	W.new<-cbind(W.new,rnorm(n.1,UX.1%*%coef,sqrt(sigma2)))

	WX.0<-cbind(rep(1,n.0),W.0)
	WX.1<-cbind(rep(1,n.1),W.new[,i+1])
	WX<-rbind(WX.0,WX.1)
	s2<-t(U.new-WX%*%solve(t(WX)%*%WX)%*%t(WX)%*%U.new)%*%(U.new-WX%*%solve(t(WX)%*%WX)%*%t(WX)%*%U.new)/(n-ncol(WX))
	sigma2<-1/rgamma(1,(n-ncol(WX))/2,(n-ncol(WX))*s2/2)
	coef<-mvrnorm(1,solve(t(WX)%*%WX)%*%t(WX)%*%U.new,solve(t(WX)%*%WX)*sigma2)
	a0<-coef[1]; a1<-coef[2]
	U.new.0<-cbind(U.new.0,rnorm(n.0,WX.0%*%coef,sqrt(sigma2)))
	U.new.1<-cbind(U.new.1,rnorm(n.1,WX.1%*%coef,sqrt(sigma2)))
}


Y.imp<-do.call(cbind, lapply(seq(100,ncol(W.new),10), function(i) c(Y.0, W.new[,i]-X.1)))
mean.tr<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.tr<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(X.0, probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(W.0, probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(Y.0, probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	X<-data$X
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	X.0<-X[m==0]
	X.1<-X[m==1]
	Y.0<-Y[m==0]
	X<-c(X.0,X.1)

	b.lamda<-function(lamda)
	{
		if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
		if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
		return(b)
	}

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.tr<-mean.tr-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.tr<-bias.tr^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.tr<-2*t*sqrt(var.tr)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.tr<-(true.mu2<(mean.tr+t*sqrt(var.tr)) & true.mu2>(mean.tr-t*sqrt(var.tr)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.tr, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.tr, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.tr, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.tr, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=Inf ####

sim6<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
X<-4*rgamma(n,1,1)
Y<-rnorm(n,10+X,1)
#U<-rnorm(n,-1.5+0.5*X,1)
#U<-rnorm(n,-2.5+0.15*(X+Y),1)
U<-rnorm(n,-3.5+0.25*Y,1)
m<-(U>0)
n.0<-sum(m==0)
n.1<-sum(m==1)
X.0<-X[m==0]
X.1<-X[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
U.0<-U[m==0]
U.1<-U[m==1]
true.mu2<-14
data<-data.frame(X=X, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n.0

#### True model ####

coef<-lm(Y.0~X.0)$coefficients
Y.new<-as.matrix(coef[1]+coef[2]*X.1)
a0<-qnorm(mean(m),0,1)-mean(c(Y.0,Y.new))
a1<-1
sigma2<-1
U.new.0<-as.matrix(rnorm(n.0,a0+a1*Y.0,1))
U.new.1<-as.matrix(rnorm(n.1,a0+a1*Y.new,1))

for (i in 1:1000)
{
	j<-0
	while(max(U.new.0[,i])>0)
	{
		index<-which(U.new.0[,i]>0)
		U.new.0[,i][index]<-rnorm(length(index),a0+a1*Y.0[index],sqrt(sigma2))
		j<-j+1
		if (j==10000) {U.new.0[,i][U.new.0[,i]>0]<-0}
	}

	j<-0
	while(min(U.new.1[,i])<0)
	{
		index<-which(U.new.1[,i]<0)
		U.new.1[,i][index]<-rnorm(length(index),a0+a1*Y.new[,i][index],sqrt(sigma2))
		j<-j+1
		if (j==10000) {U.new.1[,i][U.new.1[,i]<0]<-0}
	}

	U.new<-c(U.new.0[,i],U.new.1[,i])

	UX.0<-cbind(rep(1,n.0),U.new.0[,i],X.0)
	UX.1<-cbind(rep(1,n.1),U.new.1[,i],X.1)
	s2<-t(Y.0-UX.0%*%solve(t(UX.0)%*%UX.0)%*%t(UX.0)%*%Y.0)%*%(Y.0-UX.0%*%solve(t(UX.0)%*%UX.0)%*%t(UX.0)%*%Y.0)/(n.0-ncol(UX.0))
	sigma2<-1/rgamma(1,(n.0-ncol(UX.0))/2,(n.0-ncol(UX.0))*s2/2)
	coef<-mvrnorm(1,solve(t(UX.0)%*%UX.0)%*%t(UX.0)%*%Y.0,solve(t(UX.0)%*%UX.0)*sigma2)
	Y.new<-cbind(Y.new,rnorm(n.1,UX.1%*%coef,sqrt(sigma2)))

	YX.0<-cbind(rep(1,n.0),Y.0)
	YX.1<-cbind(rep(1,n.1),Y.new[,i+1])
	YX<-rbind(YX.0,YX.1)
	s2<-t(U.new-YX%*%solve(t(YX)%*%YX)%*%t(YX)%*%U.new)%*%(U.new-YX%*%solve(t(YX)%*%YX)%*%t(YX)%*%U.new)/(n-ncol(YX))
	sigma2<-1/rgamma(1,(n-ncol(YX))/2,(n-ncol(YX))*s2/2)
	coef<-mvrnorm(1,solve(t(YX)%*%YX)%*%t(YX)%*%U.new,solve(t(YX)%*%YX)*sigma2)
	a0<-coef[1]; a1<-coef[2]
	U.new.0<-cbind(U.new.0,rnorm(n.0,YX.0%*%coef,sqrt(sigma2)))
	U.new.1<-cbind(U.new.1,rnorm(n.1,YX.1%*%coef,sqrt(sigma2)))
}


Y.imp<-do.call(cbind, lapply(seq(100,ncol(Y.new),10), function(i) c(Y.0, Y.new[,i])))
mean.tr<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.tr<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(X.0, probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(W.0, probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(Y.0, probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	X<-data$X
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	X.0<-X[m==0]
	X.1<-X[m==1]
	Y.0<-Y[m==0]
	X<-c(X.0,X.1)

	b.lamda<-function(lamda)
	{
		if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
		if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
		return(b)
	}

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.tr<-mean.tr-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.tr<-bias.tr^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.tr<-2*t*sqrt(var.tr)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.tr<-(true.mu2<(mean.tr+t*sqrt(var.tr)) & true.mu2>(mean.tr-t*sqrt(var.tr)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.tr, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.tr, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.tr, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.tr, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


######################################## Scenario 3: Set of normals ########################################

#### Lamda=0 ####

sim7<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rnorm(n,0,1)
Z3<-rnorm(n,0,1)
Y<-rnorm(n,15+Z1+2*Z2+Z3,1)
true.mu2<-15
m<-rbinom(n,1,exp(-0+0.5*(Z1+2*Z2+Z3))/(1+exp(-0+0.5*(Z1+2*Z2+Z3))))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=1 ####

sim8<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rnorm(n,0,1)
Z3<-rnorm(n,0,1)
Y<-rnorm(n,15+Z1+2*Z2+Z3,1)
true.mu2<-15
m<-rbinom(n,1,exp(-3.5+0.25*(Y+0.98*Z1+1.95*Z2+0.98*Z3))/(1+exp(-3.5+0.25*(Y+0.98*Z1+1.95*Z2+0.98*Z3))))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=Inf ####

sim9<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rnorm(n,0,1)
Z3<-rnorm(n,0,1)
Y<-rnorm(n,15+Z1+2*Z2+Z3,1)
true.mu2<-15
m<-rbinom(n,1,exp(-7.5+0.5*Y)/(1+exp(-7.5+0.5*Y)))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=None ####

sim10<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rnorm(n,0,1)
Z3<-rnorm(n,0,1)
Y<-rnorm(n,15+Z1+2*Z2+Z3,1)
true.mu2<-15
m<-rbinom(n,1,exp(-0+2*(Z2))/(1+exp(-0+2*(Z2))))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=None ####

sim11<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rnorm(n,0,1)
Z3<-rnorm(n,0,1)
Y<-rnorm(n,15+Z1+2*Z2+Z3,1)
true.mu2<-15
m<-rbinom(n,1,exp(-7.5+0.5*(2*Z2+Y))/(1+exp(-7.5+0.5*(2*Z2+Y))))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


######################################## Scenario 4: Varying Z ########################################

#### Lamda=0 ####

sim12<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rgamma(n,1,1)
Z3<-rbinom(n,1,0.5)
Y<-rnorm(n,10+Z1+4*Z2+Z3,1)
true.mu2<-14.5
m<-rbinom(n,1,exp(-2+0.5*(Z1+4*Z2+Z3))/(1+exp(-2+0.5*(Z1+4*Z2+Z3))))
#m<-rbinom(n,1,exp(-4.5+0.25*(Y+0.98*Z1+3.9*Z2+0.98*Z3))/(1+exp(-4.5+0.25*(Y+0.98*Z1+3.9*Z2+0.98*Z3))))
#m<-rbinom(n,1,exp(-7+0.5*Y)/(1+exp(-7+0.5*Y)))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=1 ####

sim13<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rgamma(n,1,1)
Z3<-rbinom(n,1,0.5)
Y<-rnorm(n,10+Z1+4*Z2+Z3,1)
true.mu2<-14.5
#m<-rbinom(n,1,exp(-2+0.5*(Z1+4*Z2+Z3))/(1+exp(-2+0.5*(Z1+4*Z2+Z3))))
m<-rbinom(n,1,exp(-4.5+0.25*(Y+0.98*Z1+3.9*Z2+0.98*Z3))/(1+exp(-4.5+0.25*(Y+0.98*Z1+3.9*Z2+0.98*Z3))))
#m<-rbinom(n,1,exp(-7+0.5*Y)/(1+exp(-7+0.5*Y)))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=Inf ####

sim14<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rgamma(n,1,1)
Z3<-rbinom(n,1,0.5)
Y<-rnorm(n,10+Z1+4*Z2+Z3,1)
true.mu2<-14.5
#m<-rbinom(n,1,exp(-2+0.5*(Z1+4*Z2+Z3))/(1+exp(-2+0.5*(Z1+4*Z2+Z3))))
#m<-rbinom(n,1,exp(-4.5+0.25*(Y+0.98*Z1+3.9*Z2+0.98*Z3))/(1+exp(-4.5+0.25*(Y+0.98*Z1+3.9*Z2+0.98*Z3))))
m<-rbinom(n,1,exp(-7+0.5*Y)/(1+exp(-7+0.5*Y)))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=None ####

sim15<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rgamma(n,1,1)
Z3<-rbinom(n,1,0.5)
Y<-rnorm(n,10+Z1+4*Z2+Z3,1)
true.mu2<-14.5
m<-rbinom(n,1,exp(-1+(Z2))/(1+exp(-1+(Z2))))
#m<-rbinom(n,1,exp(-4+0.25*(Z2+Y))/(1+exp(-4+0.25*(Z2+Y))))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=None ####

sim16<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rgamma(n,1,1)
Z3<-rbinom(n,1,0.5)
Y<-rnorm(n,10+Z1+4*Z2+Z3,1)
true.mu2<-14.5
#m<-rbinom(n,1,exp(-1+(Z2))/(1+exp(-1+(Z2))))
m<-rbinom(n,1,exp(-4+0.25*(Z2+Y))/(1+exp(-4+0.25*(Z2+Y))))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z3.0)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z3.1)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


######################################## Scenario 5: Set of normals with quadratic ########################################

#### Lamda=0 ####

sim17<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rnorm(n,0,1)
Z3<-rnorm(n,0,1)
Y<-rnorm(n,10+Z1+Z2+2*Z2^2,1)
true.mu2<-12
m<-rbinom(n,1,exp(-1+0.5*(Z1+Z2+2*Z2^2))/(1+exp(-1+0.5*(Z1+Z2+2*Z2^2))))
#m<-rbinom(n,1,exp(-3+0.25*(Y+0.97*Z1+0.97*Z2+1.95*Z2^2))/(1+exp(-3+0.25*(Y+0.97*Z1+0.97*Z2+1.95*Z2^2))))
#m<-rbinom(n,1,exp(-6+0.5*Y)/(1+exp(-6+0.5*Y)))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z2.0^2)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z2.1^2)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z2.0^2)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z2.1^2)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=1 ####

sim18<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rnorm(n,0,1)
Z3<-rnorm(n,0,1)
Y<-rnorm(n,10+Z1+Z2+2*Z2^2,1)
true.mu2<-12
#m<-rbinom(n,1,exp(-1+0.5*(Z1+Z2+2*Z2^2))/(1+exp(-1+0.5*(Z1+Z2+2*Z2^2))))
m<-rbinom(n,1,exp(-3+0.25*(Y+0.97*Z1+0.97*Z2+1.95*Z2^2))/(1+exp(-3+0.25*(Y+0.97*Z1+0.97*Z2+1.95*Z2^2))))
#m<-rbinom(n,1,exp(-6+0.5*Y)/(1+exp(-6+0.5*Y)))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z2.0^2)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z2.1^2)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z2.0^2)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z2.1^2)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=Inf ####

sim19<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rnorm(n,0,1)
Z3<-rnorm(n,0,1)
Y<-rnorm(n,10+Z1+Z2+2*Z2^2,1)
true.mu2<-12
#m<-rbinom(n,1,exp(-1+0.5*(Z1+Z2+2*Z2^2))/(1+exp(-1+0.5*(Z1+Z2+2*Z2^2))))
#m<-rbinom(n,1,exp(-3+0.25*(Y+0.97*Z1+0.97*Z2+1.95*Z2^2))/(1+exp(-3+0.25*(Y+0.97*Z1+0.97*Z2+1.95*Z2^2))))
m<-rbinom(n,1,exp(-6+0.5*Y)/(1+exp(-6+0.5*Y)))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z2.0^2)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z2.1^2)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z2.0^2)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z2.1^2)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=None ####

sim20<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rnorm(n,0,1)
Z3<-rnorm(n,0,1)
Y<-rnorm(n,10+Z1+Z2+2*Z2^2,1)
true.mu2<-12
m<-rbinom(n,1,exp(-0+4*Z2)/(1+exp(-0+4*Z2)))
#m<-rbinom(n,1,exp(-5.5+0.5*(4*Z2+Y))/(1+exp(-5.5+0.5*(4*Z2+Y))))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z2.0^2)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z2.1^2)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z2.0^2)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z2.1^2)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=None ####

sim21<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rnorm(n,0,1)
Z3<-rnorm(n,0,1)
Y<-rnorm(n,10+Z1+Z2+2*Z2^2,1)
true.mu2<-12
#m<-rbinom(n,1,exp(-0+4*Z2)/(1+exp(-0+4*Z2)))
m<-rbinom(n,1,exp(-5.5+0.5*(4*Z2+Y))/(1+exp(-5.5+0.5*(4*Z2+Y))))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z2.0^2)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z2.1^2)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z2.0^2)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z2.1^2)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


######################################## Scenario 6: Set of normals with interaction ########################################

#### Lamda=0 ####

sim22<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rnorm(n,0,1)
Z3<-rnorm(n,0,1)
Y<-rnorm(n,20+Z1+Z2+2*Z1*Z2,1)
true.mu2<-20
m<-rbinom(n,1,exp(-0+(Z1+Z2+2*Z1*Z2))/(1+exp(-0+(Z1+Z2+2*Z1*Z2))))
#m<-rbinom(n,1,exp(-5+0.25*(Y+0.98*Z1+0.98*Z2+1.96*Z1*Z2))/(1+exp(-5+0.25*(Y+0.98*Z1+0.98*Z2+1.96*Z1*Z2))))
#m<-rbinom(n,1,exp(-10+0.5*Y)/(1+exp(-10+0.5*Y)))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z1.0*Z2.0)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z1.1*Z2.1)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z1.0*Z2.0)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z1.1*Z2.1)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=1 ####

sim23<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rnorm(n,0,1)
Z3<-rnorm(n,0,1)
Y<-rnorm(n,20+Z1+Z2+2*Z1*Z2,1)
true.mu2<-20
#m<-rbinom(n,1,exp(-0+(Z1+Z2+2*Z1*Z2))/(1+exp(-0+(Z1+Z2+2*Z1*Z2))))
m<-rbinom(n,1,exp(-5+0.25*(Y+0.98*Z1+0.98*Z2+1.96*Z1*Z2))/(1+exp(-5+0.25*(Y+0.98*Z1+0.98*Z2+1.96*Z1*Z2))))
#m<-rbinom(n,1,exp(-10+0.5*Y)/(1+exp(-10+0.5*Y)))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z1.0*Z2.0)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z1.1*Z2.1)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z1.0*Z2.0)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z1.1*Z2.1)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=Inf ####

sim24<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rnorm(n,0,1)
Z3<-rnorm(n,0,1)
Y<-rnorm(n,20+Z1+Z2+2*Z1*Z2,1)
true.mu2<-20
#m<-rbinom(n,1,exp(-0+(Z1+Z2+2*Z1*Z2))/(1+exp(-0+(Z1+Z2+2*Z1*Z2))))
#m<-rbinom(n,1,exp(-5+0.25*(Y+0.98*Z1+0.98*Z2+1.96*Z1*Z2))/(1+exp(-5+0.25*(Y+0.98*Z1+0.98*Z2+1.96*Z1*Z2))))
m<-rbinom(n,1,exp(-10+0.5*Y)/(1+exp(-10+0.5*Y)))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z1.0*Z2.0)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z1.1*Z2.1)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z1.0*Z2.0)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z1.1*Z2.1)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=None ####

sim25<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rnorm(n,0,1)
Z3<-rnorm(n,0,1)
Y<-rnorm(n,20+Z1+Z2+2*Z1*Z2,1)
true.mu2<-20
m<-rbinom(n,1,exp(-0+(5*Z2))/(1+exp(-0+(5*Z2))))
#m<-rbinom(n,1,exp(-10+0.5*(5*Z2+Y))/(1+exp(-10+0.5*(5*Z2+Y))))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z1.0*Z2.0)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z1.1*Z2.1)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z1.0*Z2.0)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z1.1*Z2.1)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}


#### Lamda=None ####

sim26<-function(j){

#Load functions
source("functions.R")

#Generate data
n<-400
#n<-100
Z1<-rnorm(n,0,1)
Z2<-rnorm(n,0,1)
Z3<-rnorm(n,0,1)
Y<-rnorm(n,20+Z1+Z2+2*Z1*Z2,1)
true.mu2<-20
#m<-rbinom(n,1,exp(-0+(5*Z2))/(1+exp(-0+(5*Z2))))
m<-rbinom(n,1,exp(-10+0.5*(5*Z2+Y))/(1+exp(-10+0.5*(5*Z2+Y))))
Z1.0<-Z1[m==0]
Z1.1<-Z1[m==1]
Z2.0<-Z2[m==0]
Z2.1<-Z2[m==1]
Z3.0<-Z3[m==0]
Z3.1<-Z3[m==1]
Y.0<-Y[m==0]
Y.1<-Y[m==1]
n.0<-sum(m==0)
n.1<-sum(m==1)
Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z1.0*Z2.0)
Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z1.1*Z2.1)
X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X<-c(X.0,X.1)
data<-data.frame(Z1=Z1, Z2=Z2, Z3=Z3, Y=Y, m=m)


#### BD & CC ####

mean.bd<-mean(Y)
var.bd<-var(Y)/n
mean.cc<-mean(Y.0)
var.cc<-var(Y.0)/n

#### PSPP homoscedastic errors ####

q<-5*(n==400)+2*(n==100)

#### Lamda=0 ####

knots<-quantile(c(min(X.0),max(X.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (X.0-knots[i+1])*(X.0>knots[i+1]))),nrow=n.0,ncol=q)
X_0<-cbind(rep(1,n.0),X.0,knot.0)

knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (X.1-knots[i+1])*(X.1>knots[i+1]))),nrow=n.1,ncol=q)	
X_1<-cbind(rep(1,n.1),X.1,knot.1)

estimates<-pspp.hom(X_0, Y.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-do.call(cbind, lapply(1:ncol(coef), function(i) rnorm(n.1, X_1%*%coef[,i], sqrt(sigma2[i]*X_1[,ncol(X_1)-q]^(2*alpha[i])))))
Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.0<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=1 ####

#### PSPP by bivariate apporoximation of donor pool ####

W.0<-X.0+Y.0
knots<-quantile(c(min(W.0),max(W.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (W.0-knots[i+1])*(W.0>knots[i+1]))),nrow=n.0,ncol=q)
W_0<-cbind(rep(1,n.0),W.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, 1, 100)
W.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (W.p-knots[i+1])*(W.p>knots[i+1]))),nrow=length(W.p),ncol=q)	
W_1<-cbind(rep(1,length(W.p)),W.p,knot.1)

estimates<-pspp.hom(W_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(W.p), W_1%*%coef[,10*j], sqrt(sigma2[10*j]*W_1[,ncol(W_1)-q]^(2*alpha[10*j]))), W.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i]-X.1)))
mean.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.1<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### Lamda=Inf ####

#### PSPP by bivariate apporoximation of donor pool ####

knots<-quantile(c(min(Y.0),max(Y.0)), probs=seq(0,1,1/(q+1)))
knot.0<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.0-knots[i+1])*(Y.0>knots[i+1]))),nrow=n.0,ncol=q)
Y_0<-cbind(rep(1,n.0),Y.0,knot.0)

par<-estimate3(X.0, X.1, Y.0, Inf, 100)
Y.p<-unlist(lapply(1:n.1, function(i) (X.1[i]-par[,1])/par[,2]))
knot.1<-matrix(do.call(cbind, lapply(1:q, function(i) (Y.p-knots[i+1])*(Y.p>knots[i+1]))),nrow=length(Y.p),ncol=q)	
Y_1<-cbind(rep(1,length(Y.p)),Y.p,knot.1)

estimates<-pspp.hom(Y_0, X.0, q, 2000)
coef<-estimates$BETA
sigma2<-estimates$SIGMA2
alpha<-estimates$ALPHA

imp<-NULL
for (j in 1:100)
{
	joint<-cbind(rnorm(length(Y.p), Y_1%*%coef[,10*j], sqrt(sigma2[10*j]*Y_1[,ncol(Y_1)-q]^(2*alpha[10*j]))), Y.p)
	index<-unlist(lapply(1:n.1, function(i) which(abs(X.1[i]-joint[,1])==min(abs(X.1[i]-joint[,1])))))
	imp<-cbind(imp, joint[index,2])
}

Y.imp<-do.call(cbind, lapply(1:ncol(imp), function(i) c(Y.0, imp[,i])))
mean.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))
var.ps.n<-mean(unlist(lapply(1:ncol(Y.imp), function(i) var(Y.imp[,i])/n)))+(ncol(Y.imp)+1)/ncol(Y.imp)*var(unlist(lapply(1:ncol(Y.imp), function(i) mean(Y.imp[,i]))))


#### ML PMM ####

b.lamda<-function(lamda)
{
	if (lamda==0) {b<-(lamda*var(Y.0)+cov(X.0,Y.0))/(lamda*cov(X.0,Y.0)+var(X.0))}
	if (lamda>0) {b<-(var(Y.0)+cov(X.0,Y.0)/lamda)/(cov(X.0,Y.0)+var(X.0)/lamda)}
	return(b)
}

boot<-function(data)
{
	Z1<-data$Z1
	Z2<-data$Z2
	Z3<-data$Z3
	Y<-data$Y
	m<-data$m
	n.0<-sum(m==0)
	n.1<-sum(m==1)
	Z1.0<-Z1[m==0]
	Z1.1<-Z1[m==1]
	Z2.0<-Z2[m==0]
	Z2.1<-Z2[m==1]
	Z3.0<-Z3[m==0]
	Z3.1<-Z3[m==1]
	Y.0<-Y[m==0]

	##########CORR##########

	Z.0<-cbind(rep(1,n.0),Z1.0,Z2.0,Z1.0*Z2.0)
	Z.1<-cbind(rep(1,n.1),Z1.1,Z2.1,Z1.1*Z2.1)
	X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
	X<-c(X.0,X.1)

	mu2.corr.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
	mu2.corr.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
	mu2.corr.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))

	return(c(mu2.corr.0, mu2.corr.1, mu2.corr.n))
}

B<-500
resamples<-do.call(cbind, lapply(1:B, function(i) sample(1:n, replace = T)))
estimates<-do.call(rbind, lapply(1:B, function(i) boot(data[resamples[,i],])))
mean.ml.0<-mean(Y.0)+b.lamda(0)*(mean(X)-mean(X.0))
mean.ml.1<-mean(Y.0)+b.lamda(1)*(mean(X)-mean(X.0))
mean.ml.n<-mean(Y.0)+b.lamda(Inf)*(mean(X)-mean(X.0))
var.ml.0<-var(estimates[,1])
var.ml.1<-var(estimates[,2])
var.ml.n<-var(estimates[,3])


#### BC ####

X.0<-Z.0%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
X.1<-Z.1%*%solve(t(Z.0)%*%Z.0)%*%t(Z.0)%*%Y.0
mu2.bc.lamda.0<-estimate(X.0, X.1, Y.0, 0, 1000)
mu2.bc.lamda.1<-estimate(X.0, X.1, Y.0, 1, 1000)
mu2.bc.lamda.n<-estimate(X.0, X.1, Y.0, Inf, 1000)


t<-qt(0.975, n-1)
bias.bd<-mean.bd-true.mu2
bias.cc<-mean.cc-true.mu2
bias.ps.0<-mean.ps.0-true.mu2
bias.ml.0<-mean.ml.0-true.mu2
bias.bc.0<-median(mu2.bc.lamda.0)-true.mu2
bias.ps.1<-mean.ps.1-true.mu2
bias.ml.1<-mean.ml.1-true.mu2
bias.bc.1<-median(mu2.bc.lamda.1)-true.mu2
bias.ps.n<-mean.ps.n-true.mu2
bias.ml.n<-mean.ml.n-true.mu2
bias.bc.n<-median(mu2.bc.lamda.n)-true.mu2
rmse.bd<-bias.bd^2
rmse.cc<-bias.cc^2
rmse.ps.0<-bias.ps.0^2
rmse.ml.0<-bias.ml.0^2
rmse.bc.0<-bias.bc.0^2
rmse.ps.1<-bias.ps.1^2
rmse.ml.1<-bias.ml.1^2
rmse.bc.1<-bias.bc.1^2
rmse.ps.n<-bias.ps.n^2
rmse.ml.n<-bias.ml.n^2
rmse.bc.n<-bias.bc.n^2
ciw.bd<-2*t*sqrt(var.bd)
ciw.cc<-2*t*sqrt(var.cc)
ciw.ps.0<-2*t*sqrt(var.ps.0)
ciw.ml.0<-2*t*sqrt(var.ml.0)
ciw.bc.0<-quantile(mu2.bc.lamda.0,.975)-quantile(mu2.bc.lamda.0,.025)
ciw.ps.1<-2*t*sqrt(var.ps.1)
ciw.ml.1<-2*t*sqrt(var.ml.1)
ciw.bc.1<-quantile(mu2.bc.lamda.1,.975)-quantile(mu2.bc.lamda.1,.025)
ciw.ps.n<-2*t*sqrt(var.ps.n)
ciw.ml.n<-2*t*sqrt(var.ml.n)
ciw.bc.n<-quantile(mu2.bc.lamda.n,.975)-quantile(mu2.bc.lamda.n,.025)
cov.bd<-(true.mu2<(mean.bd+t*sqrt(var.bd)) & true.mu2>(mean.bd-t*sqrt(var.bd)))
cov.cc<-(true.mu2<(mean.cc+t*sqrt(var.cc)) & true.mu2>(mean.cc-t*sqrt(var.cc)))
cov.ps.0<-(true.mu2<(mean.ps.0+t*sqrt(var.ps.0)) & true.mu2>(mean.ps.0-t*sqrt(var.ps.0)))
cov.ml.0<-(true.mu2<(mean.ml.0+t*sqrt(var.ml.0)) & true.mu2>(mean.ml.0-t*sqrt(var.ml.0)))
cov.bc.0<-(true.mu2<quantile(mu2.bc.lamda.0,.975) & true.mu2>quantile(mu2.bc.lamda.0,.025))
cov.ps.1<-(true.mu2<(mean.ps.1+t*sqrt(var.ps.1)) & true.mu2>(mean.ps.1-t*sqrt(var.ps.1)))
cov.ml.1<-(true.mu2<(mean.ml.1+t*sqrt(var.ml.1)) & true.mu2>(mean.ml.1-t*sqrt(var.ml.1)))
cov.bc.1<-(true.mu2<quantile(mu2.bc.lamda.1,.975) & true.mu2>quantile(mu2.bc.lamda.1,.025))
cov.ps.n<-(true.mu2<(mean.ps.n+t*sqrt(var.ps.n)) & true.mu2>(mean.ps.n-t*sqrt(var.ps.n)))
cov.ml.n<-(true.mu2<(mean.ml.n+t*sqrt(var.ml.n)) & true.mu2>(mean.ml.n-t*sqrt(var.ml.n)))
cov.bc.n<-(true.mu2<quantile(mu2.bc.lamda.n,.975) & true.mu2>quantile(mu2.bc.lamda.n,.025))

return(c(bias.bd, bias.cc, bias.ps.0, bias.ml.0, bias.bc.0, bias.ps.1, bias.ml.1, bias.bc.1, bias.ps.n, bias.ml.n, bias.bc.n, 
	 rmse.bd, rmse.cc, rmse.ps.0, rmse.ml.0, rmse.bc.0, rmse.ps.1, rmse.ml.1, rmse.bc.1, rmse.ps.n, rmse.ml.n, rmse.bc.n, 
	 ciw.bd, ciw.cc, ciw.ps.0, ciw.ml.0, ciw.bc.0, ciw.ps.1, ciw.ml.1, ciw.bc.1, ciw.ps.n, ciw.ml.n, ciw.bc.n, 
	 cov.bd, cov.cc, cov.ps.0, cov.ml.0, cov.bc.0, cov.ps.1, cov.ml.1, cov.bc.1, cov.ps.n, cov.ml.n, cov.bc.n))

}



#### Simulate scenarios ####

#Initialize results
results.1<-results.2<-results.3<-results.4<-results.5<-results.6<-results.7<-results.8<-results.9<-results.10<-
results.11<-results.12<-results.13<-results.14<-results.15<-results.16<-results.17<-results.18<-results.19<-
results.20<-results.21<-results.22<-results.23<-results.24<-results.25<-results.26<-NULL

#Save results to file
#file<-c("simulation_400.RData")
#file<-c("simulation_100.RData")
file<-c("simulation_test.RData")


#Perform simulation
while (length(results.1)<M) {results.1<-c(results.1, sfLapply(1:cpus, sim1)); save.image(file)}
while (length(results.2)<M) {results.2<-c(results.2, sfLapply(1:cpus, sim2)); save.image(file)}
while (length(results.3)<M) {results.3<-c(results.3, sfLapply(1:cpus, sim3)); save.image(file)}
while (length(results.4)<M) {results.4<-c(results.4, sfLapply(1:cpus, sim4)); save.image(file)}
while (length(results.5)<M) {results.5<-c(results.5, sfLapply(1:cpus, sim5)); save.image(file)}
while (length(results.6)<M) {results.6<-c(results.6, sfLapply(1:cpus, sim6)); save.image(file)}
while (length(results.7)<M) {results.7<-c(results.7, sfLapply(1:cpus, sim7)); save.image(file)}
while (length(results.8)<M) {results.8<-c(results.8, sfLapply(1:cpus, sim8)); save.image(file)}
while (length(results.9)<M) {results.9<-c(results.9, sfLapply(1:cpus, sim9)); save.image(file)}
while (length(results.10)<M) {results.10<-c(results.10, sfLapply(1:cpus, sim10)); save.image(file)}
while (length(results.11)<M) {results.11<-c(results.11, sfLapply(1:cpus, sim11)); save.image(file)}
while (length(results.12)<M) {results.12<-c(results.12, sfLapply(1:cpus, sim12)); save.image(file)}
while (length(results.13)<M) {results.13<-c(results.13, sfLapply(1:cpus, sim13)); save.image(file)}
while (length(results.14)<M) {results.14<-c(results.14, sfLapply(1:cpus, sim14)); save.image(file)}
while (length(results.15)<M) {results.15<-c(results.15, sfLapply(1:cpus, sim15)); save.image(file)}
while (length(results.16)<M) {results.16<-c(results.16, sfLapply(1:cpus, sim16)); save.image(file)}
while (length(results.17)<M) {results.17<-c(results.17, sfLapply(1:cpus, sim17)); save.image(file)}
while (length(results.18)<M) {results.18<-c(results.18, sfLapply(1:cpus, sim18)); save.image(file)}
while (length(results.19)<M) {results.19<-c(results.19, sfLapply(1:cpus, sim19)); save.image(file)}
while (length(results.20)<M) {results.20<-c(results.20, sfLapply(1:cpus, sim20)); save.image(file)}
while (length(results.21)<M) {results.21<-c(results.21, sfLapply(1:cpus, sim21)); save.image(file)}
while (length(results.22)<M) {results.22<-c(results.22, sfLapply(1:cpus, sim22)); save.image(file)}
while (length(results.23)<M) {results.23<-c(results.23, sfLapply(1:cpus, sim23)); save.image(file)}
while (length(results.24)<M) {results.24<-c(results.24, sfLapply(1:cpus, sim24)); save.image(file)}
while (length(results.25)<M) {results.25<-c(results.25, sfLapply(1:cpus, sim25)); save.image(file)}
while (length(results.26)<M) {results.26<-c(results.26, sfLapply(1:cpus, sim26)); save.image(file)}

#Output results and assign column names
results1<-do.call(rbind, results.1)
final1<-colMeans(results1)
final1[12:22]<-sqrt(final1[12:22])
names(final1)<-names
round(1000*final1)

results2<-do.call(rbind, results.2)
final2<-colMeans(results2)
final2[12:22]<-sqrt(final2[12:22])
names(final2)<-names
round(1000*final2)

results3<-do.call(rbind, results.3)
final3<-colMeans(results3)
final3[12:22]<-sqrt(final3[12:22])
names(final3)<-names
round(1000*final3)

results4<-do.call(rbind, results.4)
final4<-colMeans(results4)
final4[13:24]<-sqrt(final4[13:24])
names(final4)<-names.tr
round(1000*final4)

results5<-do.call(rbind, results.5)
final5<-colMeans(results5)
final5[13:24]<-sqrt(final5[13:24])
names(final5)<-names.tr
round(1000*final5)

results6<-do.call(rbind, results.6)
final6<-colMeans(results6)
final6[13:24]<-sqrt(final6[13:24])
names(final6)<-names.tr
round(1000*final6)

results7<-do.call(rbind, results.7)
final7<-colMeans(results7)
final7[12:22]<-sqrt(final7[12:22])
names(final7)<-names
round(1000*final7)

results8<-do.call(rbind, results.8)
final8<-colMeans(results8)
final8[12:22]<-sqrt(final8[12:22])
names(final8)<-names
round(1000*final8)

results9<-do.call(rbind, results.9)
final9<-colMeans(results9)
final9[12:22]<-sqrt(final9[12:22])
names(final9)<-names;
round(1000*final9)

results10<-do.call(rbind, results.10)
final10<-colMeans(results10)
final10[12:22]<-sqrt(final10[12:22])
names(final10)<-names
round(1000*final10)

results11<-do.call(rbind, results.11)
final11<-colMeans(results11)
final11[12:22]<-sqrt(final11[12:22])
names(final11)<-names
round(1000*final11)

results12<-do.call(rbind, results.12)
final12<-colMeans(results12)
final12[12:22]<-sqrt(final12[12:22])
names(final12)<-names
round(1000*final12)

results13<-do.call(rbind, results.13)
final13<-colMeans(results13, na.rm=1)
final13[12:22]<-sqrt(final13[12:22])
names(final13)<-names
round(1000*final13)

results14<-do.call(rbind, results.14)
final14<-colMeans(results14)
final14[12:22]<-sqrt(final14[12:22])
names(final14)<-names
round(1000*final14)

results15<-do.call(rbind, results.15)
final15<-colMeans(results15)
final15[12:22]<-sqrt(final15[12:22])
names(final15)<-names
round(1000*final15)

results16<-do.call(rbind, results.16)
final16<-colMeans(results16)
final16[12:22]<-sqrt(final16[12:22])
names(final16)<-names
round(1000*final16)

results17<-do.call(rbind, results.17)
final17<-colMeans(results17)
final17[12:22]<-sqrt(final17[12:22])
names(final17)<-names
round(1000*final17)

results18<-do.call(rbind, results.18)
final18<-colMeans(results18)
final18[12:22]<-sqrt(final18[12:22])
names(final18)<-names
round(1000*final18)

results19<-do.call(rbind, results.19)
final19<-colMeans(results19)
final19[12:22]<-sqrt(final19[12:22])
names(final19)<-names;
round(1000*final19)

results20<-do.call(rbind, results.20)
final20<-colMeans(results20)
final20[12:22]<-sqrt(final20[12:22])
names(final20)<-names
round(1000*final20)

results21<-do.call(rbind, results.21)
final21<-colMeans(results21)
final21[12:22]<-sqrt(final21[12:22])
names(final21)<-names
round(1000*final21)

results22<-do.call(rbind, results.22)
final22<-colMeans(results22)
final22[12:22]<-sqrt(final22[12:22])
names(final22)<-names
round(1000*final22)

results23<-do.call(rbind, results.23)
final23<-colMeans(results23, na.rm=1)
final23[12:22]<-sqrt(final23[12:22])
names(final23)<-names
round(1000*final23)

results24<-do.call(rbind, results.24)
final24<-colMeans(results24)
final24[12:22]<-sqrt(final24[12:22])
names(final24)<-names
round(1000*final24)

results25<-do.call(rbind, results.25)
final25<-colMeans(results25)
final25[12:22]<-sqrt(final25[12:22])
names(final25)<-names
round(1000*final25)

results26<-do.call(rbind, results.26)
final26<-colMeans(results26)
final26[12:22]<-sqrt(final26[12:22])
names(final26)<-names
round(1000*final26)


#### Plot results ####

colors=c("blue","red","green","gray")
colors.tr=c("blue","red","green","orange","gray")
barnames=c(expression(lambda[T]*"=Inf"),expression(lambda[T]*"=1"),expression(lambda[T]*"=0"))

#Scenario 1
par(las=1)
par(mar=c(5,4,3,1))
par(mfrow=c(2,2))
par(oma = c(1, 1, 1, 1))
bias0<-abs(c(final1[5],final1[4],final1[3],final1[2]))
bias1<-abs(c(final2[8],final2[7],final2[6],final2[2]))
bias8<-abs(c(final3[11],final3[10],final3[9],final3[2]))
rmse0<-abs(c(final1[16],final1[15],final1[14],final1[13]))
rmse1<-abs(c(final2[19],final2[18],final2[17],final2[13]))
rmse8<-abs(c(final3[22],final3[21],final3[20],final3[13]))
ciw0<-abs(c(final1[27],final1[26],final1[25],final1[24]))
ciw1<-abs(c(final2[30],final2[29],final2[28],final2[24]))
ciw8<-abs(c(final3[33],final3[32],final3[31],final3[24]))
cov0<-abs(c(final1[38],final1[37],final1[36],final1[35]))
cov1<-abs(c(final2[41],final2[40],final2[39],final2[35]))
cov8<-abs(c(final3[44],final3[43],final3[42],final3[35]))
barplot(cbind(bias8,bias1,bias0), beside=T, horiz=T, xlim=c(0,1), col=colors, names.arg=barnames, main="BIAS")
barplot(cbind(rmse8,rmse1,rmse0), beside=T, horiz=T, xlim=c(0,1), col=colors, names.arg=barnames, main="RMSE")
barplot(cbind(ciw8,ciw1,ciw0), beside=T, horiz=T, xlim=c(0,1), col=colors, names.arg=barnames, main="CIW")
barplot(1-cbind(cov8,cov1,cov0), beside=T, horiz=T, xlim=c(0,0.2), col=colors, xpd=F, names.arg=barnames, main="NON-COVERAGE")
par(fig = c(0, 1, 0, 1), oma = c(0, 0, 0, 0), mar = c(0, 0, 0, 0), new = TRUE)
plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
legend("bottom", c("CC","S-BNPM","BNPM-ML","BNPM-BAYES"), xpd = T, bty="n", horiz = T, inset=c(0,0), lty=c(1,1,1,1), lwd=c(6,6,6,6), col=rev(colors))
#mtext(expression("Results for scenario 1 where "*lambda[A]*" = "*lambda[T]), side = 3, line = -1.5, outer = TRUE, cex=1.25)


#Scenario 2
par(las=1)
par(mar=c(5,4,3,1))
par(mfrow=c(2,2))
par(oma = c(1, 1, 1, 1))
bias0<-abs(c(final4[6],final4[5],final4[4],final4[3],final4[2]))
bias1<-abs(c(final5[9],final5[8],final5[7],final5[3],final5[2]))
bias8<-abs(c(final6[12],final6[11],final6[10],final6[3],final6[2]))
rmse0<-abs(c(final4[18],final4[17],final4[16],final4[15],final4[14]))
rmse1<-abs(c(final5[21],final5[20],final5[19],final5[15],final5[14]))
rmse8<-abs(c(final6[24],final6[23],final6[22],final6[15],final6[14]))
ciw0<-abs(c(final4[30],final4[29],final4[28],final4[27],final4[26]))
ciw1<-abs(c(final5[33],final5[32],final5[31],final5[27],final5[26]))
ciw8<-abs(c(final6[36],final6[35],final6[34],final6[27],final6[26]))
cov0<-abs(c(final4[42],final4[41],final4[40],final4[39],final4[38]))
cov1<-abs(c(final5[45],final5[44],final5[43],final5[39],final5[38]))
cov8<-abs(c(final6[48],final6[47],final6[46],final6[39],final6[38]))
barplot(cbind(bias8,bias1,bias0), beside=T, horiz=T, xlim=c(0,1), col=colors.tr, names.arg=barnames, main="BIAS")
barplot(cbind(rmse8,rmse1,rmse0), beside=T, horiz=T, xlim=c(0,0.5), col=colors.tr, names.arg=barnames, main="RMSE")
barplot(cbind(ciw8,ciw1,ciw0), beside=T, horiz=T, xlim=c(0,2), col=colors.tr, names.arg=barnames, main="CIW")
barplot(1-cbind(cov8,cov1,cov0), beside=T, horiz=T, xlim=c(0,0.3), col=colors.tr, xpd=F, names.arg=barnames, main="NON-COVERAGE")
par(fig = c(0, 1, 0, 1), oma = c(0, 0, 0, 0), mar = c(0, 0, 0, 0), new = TRUE)
plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
legend("bottom", c("CC","TRUE","S-BNPM","BNPM-ML","BNPM-BAYES"), xpd = T, bty="n", horiz = T, inset=c(0,0), lty=c(1,1,1,1,1), lwd=c(6,6,6,6,6), col=rev(colors.tr))
#mtext(expression("Results for scenario 2 where "*lambda[A]*" = "*lambda[T]), side = 3, line = -1.5, outer = TRUE, cex=1.25)


#Scenario 3
par(las=1)
par(mar=c(5,4,3,1))
par(mfrow=c(2,2))
par(oma = c(1, 1, 1, 1))
bias0<-abs(c(final7[5],final7[4],final7[3],final7[2]))
bias1<-abs(c(final8[8],final8[7],final8[6],final8[2]))
bias8<-abs(c(final9[11],final9[10],final9[9],final9[2]))
rmse0<-abs(c(final7[16],final7[15],final7[14],final7[13]))
rmse1<-abs(c(final8[19],final8[18],final8[17],final8[13]))
rmse8<-abs(c(final9[22],final9[21],final9[20],final9[13]))
ciw0<-abs(c(final7[27],final7[26],final7[25],final7[24]))
ciw1<-abs(c(final8[30],final8[29],final8[28],final8[24]))
ciw8<-abs(c(final9[33],final9[32],final9[31],final9[24]))
cov0<-abs(c(final7[38],final7[37],final7[36],final7[35]))
cov1<-abs(c(final8[41],final8[40],final8[39],final8[35]))
cov8<-abs(c(final9[44],final9[43],final9[42],final9[35]))
barplot(cbind(bias8,bias1,bias0), beside=T, horiz=T, xlim=c(0,1), col=colors, names.arg=barnames, main="BIAS")
barplot(cbind(rmse8,rmse1,rmse0), beside=T, horiz=T, xlim=c(0,1), col=colors, names.arg=barnames, main="RMSE")
barplot(cbind(ciw8,ciw1,ciw0), beside=T, horiz=T, xlim=c(0,1), col=colors, names.arg=barnames, main="CIW")
barplot(1-cbind(cov8,cov1,cov0), beside=T, horiz=T, xlim=c(0,0.2), col=colors, xpd=F, names.arg=barnames, main="NON-COVERAGE")
par(fig = c(0, 1, 0, 1), oma = c(0, 0, 0, 0), mar = c(0, 0, 0, 0), new = TRUE)
plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
legend("bottom", c("CC","S-PPMA","PPMA-ML","PPMA-BAYES"), xpd = T, bty="n", horiz = T, inset=c(0,0), lty=c(1,1,1,1), lwd=c(6,6,6,6), col=rev(colors))
#mtext(expression("Results for scenario 3 where "*lambda[A]*" = "*lambda[T]), side = 3, line = -1.5, outer = TRUE, cex=1.25)


#Scenario 4
par(las=1)
par(mar=c(5,4,3,1))
par(mfrow=c(2,2))
par(oma = c(1, 1, 1, 1))
bias0<-abs(c(final12[5],final12[4],final12[3],final12[2]))
bias1<-abs(c(final13[8],final13[7],final13[6],final13[2]))
bias8<-abs(c(final14[11],final14[10],final14[9],final14[2]))
rmse0<-abs(c(final12[16],final12[15],final12[14],final12[13]))
rmse1<-abs(c(final13[19],final13[18],final13[17],final13[13]))
rmse8<-abs(c(final14[22],final14[21],final14[20],final14[13]))
ciw0<-abs(c(final12[27],final12[26],final12[25],final12[24]))
ciw1<-abs(c(final13[30],final13[29],final13[28],final13[24]))
ciw8<-abs(c(final14[33],final14[32],final14[31],final14[24]))
cov0<-abs(c(final12[38],final12[37],final12[36],final12[35]))
cov1<-abs(c(final13[41],final13[40],final13[39],final13[35]))
cov8<-abs(c(final14[44],final14[43],final14[42],final14[35]))
barplot(cbind(bias8,bias1,bias0), beside=T, horiz=T, xlim=c(0,1), col=colors, names.arg=barnames, main="BIAS")
barplot(cbind(rmse8,rmse1,rmse0), beside=T, horiz=T, xlim=c(0,1), col=colors, names.arg=barnames, main="RMSE")
barplot(cbind(ciw8,ciw1,ciw0), beside=T, horiz=T, xlim=c(0,2), col=colors, names.arg=barnames, main="CIW")
barplot(1-cbind(cov8,cov1,cov0), beside=T, horiz=T, xlim=c(0,0.3), col=colors, xpd=F, names.arg=barnames, main="NON-COVERAGE")
par(fig = c(0, 1, 0, 1), oma = c(0, 0, 0, 0), mar = c(0, 0, 0, 0), new = TRUE)
plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
legend("bottom", c("CC","S-PPMA","PPMA-ML","PPMA-BAYES"), xpd = T, bty="n", horiz = T, inset=c(0,0), lty=c(1,1,1,1), lwd=c(6,6,6,6), col=rev(colors))
#mtext(expression("Results for scenario 4 where "*lambda[A]*" = "*lambda[T]), side = 3, line = -1.5, outer = TRUE, cex=1.25)


#Scenario 5
par(las=1)
par(mar=c(5,4,3,1))
par(mfrow=c(2,2))
par(oma = c(1, 1, 1, 1))
bias0<-abs(c(final17[5],final17[4],final17[3],final17[2]))
bias1<-abs(c(final18[8],final18[7],final18[6],final18[2]))
bias8<-abs(c(final19[11],final19[10],final19[9],final19[2]))
rmse0<-abs(c(final17[16],final17[15],final17[14],final17[13]))
rmse1<-abs(c(final18[19],final18[18],final18[17],final18[13]))
rmse8<-abs(c(final19[22],final19[21],final19[20],final19[13]))
ciw0<-abs(c(final17[27],final17[26],final17[25],final17[24]))
ciw1<-abs(c(final18[30],final18[29],final18[28],final18[24]))
ciw8<-abs(c(final19[33],final19[32],final19[31],final19[24]))
cov0<-abs(c(final17[38],final17[37],final17[36],final17[35]))
cov1<-abs(c(final18[41],final18[40],final18[39],final18[35]))
cov8<-abs(c(final19[44],final19[43],final19[42],final19[35]))
barplot(cbind(bias8,bias1,bias0), beside=T, horiz=T, xlim=c(0,1), col=colors, names.arg=barnames, main="BIAS")
barplot(cbind(rmse8,rmse1,rmse0), beside=T, horiz=T, xlim=c(0,1), col=colors, names.arg=barnames, main="RMSE")
barplot(cbind(ciw8,ciw1,ciw0), beside=T, horiz=T, xlim=c(0,2), col=colors, names.arg=barnames, main="CIW")
barplot(1-cbind(cov8,cov1,cov0), beside=T, horiz=T, xlim=c(0,0.3), col=colors, xpd=F, names.arg=barnames, main="NON-COVERAGE")
par(fig = c(0, 1, 0, 1), oma = c(0, 0, 0, 0), mar = c(0, 0, 0, 0), new = TRUE)
plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
legend("bottom", c("CC","S-PPMA","PPMA-ML","PPMA-BAYES"), xpd = T, bty="n", horiz = T, inset=c(0,0), lty=c(1,1,1,1), lwd=c(6,6,6,6), col=rev(colors))
#mtext(expression("Results for scenario 5 where "*lambda[A]*" = "*lambda[T]), side = 3, line = -1.5, outer = TRUE, cex=1.25)


#Scenario 6
par(las=1)
par(mar=c(5,4,3,1))
par(mfrow=c(2,2))
par(oma = c(1, 1, 1, 1))
bias0<-abs(c(final22[5],final22[4],final22[3],final22[2]))
bias1<-abs(c(final23[8],final23[7],final23[6],final23[2]))
bias8<-abs(c(final24[11],final24[10],final24[9],final24[2]))
rmse0<-abs(c(final22[16],final22[15],final22[14],final22[13]))
rmse1<-abs(c(final23[19],final23[18],final23[17],final23[13]))
rmse8<-abs(c(final24[22],final24[21],final24[20],final24[13]))
ciw0<-abs(c(final22[27],final22[26],final22[25],final22[24]))
ciw1<-abs(c(final23[30],final23[29],final23[28],final23[24]))
ciw8<-abs(c(final24[33],final24[32],final24[31],final24[24]))
cov0<-abs(c(final22[38],final22[37],final22[36],final22[35]))
cov1<-abs(c(final23[41],final23[40],final23[39],final23[35]))
cov8<-abs(c(final24[44],final24[43],final24[42],final24[35]))
barplot(cbind(bias8,bias1,bias0), beside=T, horiz=T, xlim=c(0,1), col=colors, names.arg=barnames, main="BIAS")
barplot(cbind(rmse8,rmse1,rmse0), beside=T, horiz=T, xlim=c(0,1), col=colors, names.arg=barnames, main="RMSE")
barplot(cbind(ciw8,ciw1,ciw0), beside=T, horiz=T, xlim=c(0,1), col=colors, names.arg=barnames, main="CIW")
barplot(1-cbind(cov8,cov1,cov0), beside=T, horiz=T, xlim=c(0,0.2), col=colors, xpd=F, names.arg=barnames, main="NON-COVERAGE")
par(fig = c(0, 1, 0, 1), oma = c(0, 0, 0, 0), mar = c(0, 0, 0, 0), new = TRUE)
plot(0, 0, type = "n", bty = "n", xaxt = "n", yaxt = "n")
legend("bottom", c("CC","S-PPMA","PPMA-ML","PPMA-BAYES"), xpd = T, bty="n", horiz = T, inset=c(0,0), lty=c(1,1,1,1), lwd=c(6,6,6,6), col=rev(colors))
#mtext(expression("Results for scenario 6 where "*lambda[A]*" = "*lambda[T]), side = 3, line = -1.5, outer = TRUE, cex=1.25)













